/* Minim_lib.h
 * Entetes des fonctions exportees par Minim_lib.c
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/timeb.h>

/* --------------------------------------------------------- */
/* Variables exportees par Minim POUR LA LECTURE : */
/* --------------------------------------------------------- */

/* La boucle GL (1er redraw) n'a pas encore eu lieu */
/* Utile pour se synchroniser avec un thread...*/
volatile extern int MINIM_NOT_READY;

/* Taille de la fenetre (valide si MINIM_NOT_READY est faux) */
volatile extern int MINIM_WIDTH, MINIM_HEIGHT;

/* Le clone est en train de parler */
volatile extern int MINIM_IS_SPEAKING;

/* Date de fin de parole (au sens de ftime()) : */
/*  - effective si MINIM_IS_SPEAKING == 0 */
/*  - estimee si MINIM_IS_SPEAKING == 1 */
extern struct timeb MINIM_END_TIME; // devrait etre volatile ???

/* Centralisation des textures supplementaires */
volatile extern unsigned int *MINIM_TEX_ID;
volatile extern int MINIM_NB_TEX_PLUS;

/* --------------------------------------------------------- */
/* Variable(s) exportee(s) par Minim POUR L'ECRITURE : */
/* --------------------------------------------------------- */

/* Pour ceux qui veulent modifier directement les parametres articulatoires */
/* a leurs risques et perils (synchro, taille du tableau...) ! */
/* en utilisant MINIM_ART[0] MINIM_ART[1] ... ? */
extern float *MINIM_ART;

/* --------------------------------------------------------- */
/*  LES APPELS qui sont exportes : */
/* --------------------------------------------------------- */

/*  Les parametres de definition du clone sont communiques par le client */
/* dans un tableau, comme les recoit main() :  */
/*  - argv[0] est le nom de l'executable */
/*     ... */
/*  - argv[argc-1] est le dernier parametre (ou le nom de l'executable) */
void Minim_def_cloneV(int argc, char *argv[]);

/* OU BIEN, car il faut appeler l'un des deux... */

/* Les parametres de definition du clone */
/* sont passes par une seule chaine (comme sur la ligne de commande) */
void Minim_def_cloneS(char *arg_line);

/* **** FONCTIONS D'EXTENSION POUR LES CLIENTS **** */

/* Cette serie de fonctions permet a un client externe de profiter de GLUT
 * et de la fenetre du clone, en fournissant ses propres fonctions
 * soit pour REMPLACER, soit pour COMPLEMENTER le comportement
 * par defaut de Minim, selon la nature du vecteur. */
/* Toutes ces fonctions peuvent (et parfois doivent) etre appelees
 * avant que le clone n'ait ete active avec Minim_start_clone(). */

// Permet de complementer le idle() (Cf. doc  GLUT) de Minim
// peut aussi etre appele une fois le clone lance et affiche
void Minim_IdleFunc(void (*idle)());

// Permet de complementer le reshape() (Cf. doc GLUT) de Minim
// peut aussi etre appele une fois le clone lance et affiche
void Minim_ReshapeFunc(void (*reshape)(int x,int y));

// Permet de complementer l'initialisation de Minim :
// le callback ainsi declare sera appele avant l'affichage du clone
// et apres l'initialisation de GL. Par exemple pour charger
// textures supplementaires ou des modules GL optionnels.
void Minim_InitFunc(void (*init)());

/* Un contexte graphique simple pour display() est deja fourni,
 * de facon a ce qu'on puisse directement dessiner en 2D,
 * sans avoir a effacer, commuter l'ecran etc. */
/* L'affichage sera fait EN PLUS de ce que Minim doit afficher,
 * si ces elements (fond, clone, sliders...) sont affiches */
/*  Notament :
 *  - l'ecran est deja efface,
 *  - on trace sur le BACK_BUFFER,
 *  - les coordonnees sont dans (-1,1)x(-1x1), sans transformation.
 *  - l'ecran sera affiche (swap de buffer) par Minim apres le return()
 *  une fois affiche le clone s'il est visible et ses sliders eventuels. */
void Minim_DisplayFunc(void (*display)());

/* Cette fonction permet de
 * - REMPLACER l'affichage du fond de Minim par la fonction donnee en parametre
 * - ou de reactiver le fond par defaut en passant NULL
 * En plus, la fonction retourne un pointeur sur la fonction par defaut de Minim,
 * de sorte que le client externe peut l'appeler avant de faire son rendu.
 * Le rendu fait par le client est realise avant l'affichage eventuel du clone.
 * C'est au client de mettre en place tout ce dont il a besoin (cameras...),
 * sans polluer/ralentir le contexte GL (enable/disable SVP). */
void ( *Minim_myBackgroundFunc(void (*back_func)()) )();

/* Pour les handler suivants, les parametres sont ceux de GLUT (Cf. <GL/GLUT.h>)
 * Un pointeur de fonction NULL ne sera pas appele, et retablira le comportement par defaut
 * En plus, la fonction retourne un pointeur sur la fonction par defaut de Minim,
 * de sorte que le client externe peut l'appeler avant/apres son traitement (s'il le desire). */
/* Ces fonctions n'ont d'effet QUE SI elles sont appelees AVANT Minim_start_clone() */
void (* Minim_KeyboardFunc(void (*key)(unsigned char key,int x, int y)) )(unsigned char key,int x, int y);
void (* Minim_MouseFunc(void (*click)(int button, int state, int x, int y)) )(int button, int state, int x, int y);
void (* Minim_MotionFunc(void (*drag)(int x, int y)) )(int x, int y);
void (* Minim_PassiveMotionFunc(void (*mouse)(int x, int y)) )(int x, int y);

/* **** FIN DES FONCTIONS D'EXTENSION POUR LES CLIENTS **** */

// Permet de suspendre/re-activer le rendu du clone
// (en laissant la fenetre ouverte, contrairement a Minim_hide_clone() ).
void Minim_clone_visible(int visible);

/* Cet appel ouvrira la fenetre avec le clone, */
/* chargera les dernieres ressources (textures). */
/* Si un message a ete pre-charge, le clone commencera a parler. */
/* Cet appel lance la mainloop de GLUT, il est donc bloquant, */
/* et ne retourne jamais a l'appelant. */
/*  */
/* Precondition : Minim_def_cloneS() ou Minim_def_cloneV() */
/*   doit avoir ete appele. */
void Minim_start_clone();

/* On declare les parametres d'une nouvelle sequence, qui sera */
/* chargee et jouee "des que possible" (au prochain idle()). */
/* Si le flag est vrai, les deux fichiers seront effaces (plus tard). */
/* */
/* L'appel n'est pas bloquant, et peut etre appele depuis un client lointain. */
/* la derniere requete prend la place de la precedente. */
/* le 'delete_files' n'est pas garanti, mais il sera effectue */
/* a l'appel suivant de Minim_start_clone() ou Minim_finish_clone() */
/*  */
/* Precondition : Minim_start_clone() doit avoir ete lance  */
void Minim_run_sequence(char * art_file,char *wav_file, int delete_files);

/* Comme Minim_run_sequence, sauf que le demarrage n'est pas automatique : */
/* Attention, l'appel n'est PAS bloquant : La sequence n'est generalement pas encore chargee */
/* au retour a l'appelant, elle le sera "des que possible"  (au prochain idle()). */
void Minim_load_sequence(char * art_file,char *wav_file, int delete_files);

/* Permet de demander le 'replay' d'un message deja charge : */
/*  - soit nb=0 s'il y a eu un Minim_new_sequence() */
/*  - soit nb< nb de sequences prechargees avec l'option 'logo' */
/* Le replay commencera 'des que possible' (1 ou 2 boucles idle() de GLUT) */
void Minim_play_msg(int nb);

/* Permet d'arreter le 'replay' en cours abruptement. */
/* Le buffer audio n'est pas encore purge pour l'instant. */
/* La sequence reste en memoire. */
void Minim_stop_msg();

/* On provoque la memorisation d'une demande de fermeture de GLUT & sa fenetre. */
/* ATTENTION, cela provoquera un exit(0) de la part de GLUT ! */
/* */
/* Precondition : Minim_start_clone() doit avoir ete lance  */
/*  */
/* L'appel n'est pas bloquant, et peut etre appele depuis un client lointain. */
/* L'effet ne sera effectif qu'apres un nouveau idle(); */
/* ATTENTION a l'effet du exit(0) de GLUT sur un Thread... */
/*  */
void Minim_finish_clone();

/* Provoque le masquage futur de la fenetre du clone */
void Minim_hide_clone();

/* Provoque la reapparition future de la fenetre du clone */
void Minim_show_clone();

/* Masque ou pas l'image de fond (-back) si elle est chargee */
void Minim_show_back(int flag);

/* Permet de specifier que c'est MINIM_TEX_ID[slot] qui sera utilisee comme texture de fond */
void Minim_set_back_slot(int slot);

/* Transfert d'un buffer image vers une texture GL, identifiee par MINIM_TEX_ID[slot] */
/* Les dimensions de l'image doivent etre des puissances de 2D. */
/* La partie "utile" (tex_w,tex_h) se trouve en haut a gauche */
void Minim_raw2tex(unsigned char *img_ptr, int img_w,int img_h, int tex_w,int tex_h, int my_slot);

/* Change la taille de la fenetre du clone, */
/* soit pour la premiere ouverture (avant l'appel de Minim_start_clone()) */
/* soit pendant l'affichage (apres appel de Minim_start_clone()) */
void Minim_set_window_size(int W,int H);

/* Change la position de la fenetre du clone, */
/* soit pour la premiere ouverture (avant l'appel de Minim_start_clone()) */
/* soit pendant l'affichage (apres appel de Minim_start_clone()) */
void Minim_set_window_position(int X,int Y);

/* **************************************************************** */
/* Declaration des caracteristiques de l'ecran dans le monde reel : */

/* +------------+
 * |    Y^      |
 * |     |      |
 * | Z.  +->X   | et Z pointe en dehors de l'ecran (vers l'usager)
 * |            | Les coordonnees sont EN CENTIMETRES.
 * |            |
 * +------------+ */


/* Les points 3D du monde subissent les transformations dans cet ordre :
 *  1/ prise en compte du deplacement  :  P3D-trans_clone
 *  2/ prise en compte de l'orientation: rot_clone* (P3D-trans_clone)
 *  3/ prise en compte de l'echelle pixels: resolutions* ...
 *  et doivent donc finir dans le repere/les unites de l'ecran.
 */

// Pour la conversion des pixels (de l'ecran) en centimetres (du monde) :
void Minim_set_screen_resolution(int width_pix,float width_cm, int height_pix, float height_cm);

/* Pour les deplacements et orientations du clone dans le monde physique :
 * La translation correspond aux coordonnees (en cm dans le monde)
 *   du centre de l'ecran.
 * La matrice de rotation transforme les points du monde :
 *   rotation*P3D_du_monde -> P3D_du_clone
 * ou rot[0..2] represente la premiere ligne de la matrice.
 */
void Minim_move_screen_in_world(float rot[9], float trans[3]);

/* Pour le tracking de cibles 3D mobiles : */
/* La translation correspond aux coordonnees (en cm dans le monde) */
/*   par rapport au centre de l'ecran, dans le repere de l'ecran : */
/* La matrice de rotation transforme les points du monde :*/
/*   rotation*P3D_du_monde -> P3D_du_clone */
/* ou rot[0..2] represente la premiere ligne de la matrice */
void Minim_move_screen_from_base(float rot[9], float trans[3]);

/* On oriente partiellement la tete face a la direction cible (xt,yt,zt)
 * exprimee par rapport a la base,
 * jusqu'a un certain degre (proportion, avec 0<prop<1) */
/* Le mouvement n'est effectue que partiellement (non instantanne),
 * il faut continuer a appeler pour atteindre la cible. */
void Minim_head_direction3D(float spd,float prop, float xt,float yt,float zt);

/* On oriente le regard progressivement vers la direction (xt,yt,zt)/wt,
 * exprimee par rapport a la base,
 * avec wt=1 (pour un point reel) ou wt=0 pour l'infini (direction)
 * avec un rapport de vitesse 0<spd<1 */
/* Le mouvement n'est effectue que partiellement (non instantanne),
 * il faut continuer a appeler pour atteindre la cible. */
void Minim_eyes_direction3D(float spd,float xt,float yt,float zt,float wt);

/* On oriente la tete, comme precedemment */
/* avec une cible (wx,wy) dans l'ecran (coordonnees GL (-1,1)x(-1,1) */
/* ATTENTION ! Version pas encore rendue thread-safe (GL) */
void Minim_head_direction2D(float spd,float prop,int wx,int wy);

/* Comme precedement pour le regard, avec une cible dans l'espace ecran */
/* ATTENTION ! Version pas encore rendue thread-safe (GL) */
void Minim_eyes_direction2D(float spd,int wx,int wy);

#ifdef __cplusplus
} // extern "C" {
#endif
