#include "mytiff.h"


int LoadPixels (TIFF* file, uint32** memory, uint32* width, uint32* height)
{
  int     res;
	
  /* Get the width & height of the image */

  res = TIFFGetField (file, TIFFTAG_IMAGEWIDTH, width);
  if (res != 1) {
    printf ("Tag IMAGEWIDTH was not found in tiff file\n");
    return 1;
  }
  res = TIFFGetField (file, TIFFTAG_IMAGELENGTH, height);
  if (res != 1) {
    printf ("Tag IMAGEHEIGHT was not found in tiff file\n");
    return 1;
  }
  printf ("TIFF file has an image of size %ix%i.\n",
                                          (int)(*width), (int)(*height));

  /* Allocate memory for the pixels and load them from file */
  *memory = (uint32*)_TIFFmalloc ((*width) * (*height) * sizeof(uint32));

  if (*memory == (uint32*)NULL) {
    printf ("Could not allocate memory for image pixels.\n");
    return 1;
  }
  res = TIFFReadRGBAImage (file, *width, *height, *memory, 0);
  if (res != 1) {
    printf ("Could not load image pixels into memory\n");
    _TIFFfree ((tdata_t)(*memory));
    *memory = (uint32*)NULL;
    return 1;
  }

  return 0;
}


int SwapBuffer (uint32** memoryD, uint32* memoryS, uint32 width, uint32 height)
{
  uint32*	swap;
  uint32* 	cursS;
  uint32* 	cursD;
  uint32       	pixel;
  int  		x, y;
	
  swap = (uint32*)_TIFFmalloc (width * height * sizeof(uint32));
  if (swap == (uint32*)NULL) {
    printf ("Could not allocate swap buffer\n");
    return 1;
  }
  
  cursS = memoryS;
  cursD = swap + (height - 1) * width;
  for (y = 0; y < height; y++) {
    for (x = 0; x < width; x++) {
      pixel = *cursS;
/*      *cursD =	((pixel & 0xFF000000) >> 24) |
	((pixel & 0x00FF0000) >>  8) |
	((pixel & 0x0000FF00) <<  8) |
	((pixel & 0x000000FF) << 24);
*/
      *cursD = pixel;
      cursS++;
      cursD++;
    }
    cursD -= 2 * width;
  }
  
  *memoryD = swap;
  
  return 0;
}



int SavePixels (TIFF* file, uint32* memory, uint32 width, uint32 height)
{
  int     res;
  int     dataSize;
  
  res =   TIFFSetField (file, TIFFTAG_IMAGEWIDTH,     width);
  if (res == 1)
    res = TIFFSetField (file, TIFFTAG_IMAGELENGTH,    height);
  if (res == 1)
    res = TIFFSetField (file, TIFFTAG_BITSPERSAMPLE,  (uint16)8);
  if (res == 1)
    res = TIFFSetField (file, TIFFTAG_PHOTOMETRIC,    (uint16)2);
  if (res == 1)
    res = TIFFSetField (file, TIFFTAG_SAMPLESPERPIXEL,(uint16)4);
  if (res == 1)
    res = TIFFSetField (file, TIFFTAG_PLANARCONFIG,
                                              (uint16)PLANARCONFIG_CONTIG);
	if (res == 1) {
    dataSize = sizeof(uint32) * width * height;
    res = (int)TIFFWriteRawStrip (file, 0, memory, dataSize);
    if (res == -1)
      res = 0;
  }
  if (res == 0) {
    printf ("An error occured while trying to save TIFF file\n");
    return 1;
  }

  return 0;
}
